<?php

/**
 * Vvveb
 *
 * Copyright (C) 2022  Ziadin Givan
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 */

namespace Vvveb\Controller;

use function \Vvveb\camelToUnderscore;
use function \Vvveb\controller;
use function \Vvveb\isController;
use function \Vvveb\isModel;
use function \Vvveb\model;
use GraphQL\Error\DebugFlag;
use GraphQL\GraphQL;
use GraphQL\Language\Parser;
use GraphQL\Type\Definition\ObjectType;
use GraphQL\Type\Definition\ResolveInfo;
use GraphQL\Type\Definition\Type;
use GraphQL\Utils\AST;
use GraphQL\Utils\BuildSchema;

class PostConnection {
	public $nodes;

	public $PageInfo = [];
}

#[\AllowDynamicProperties]
class Index extends Base {
	protected $stack = [];

	protected $queryIndex = -1;

	//protected $currentQuery = -1;
	protected $stackIndex = 0;

	protected $schema = [];
	
	protected $args = [];
	
	protected $returnIsList = false;

	function cascadeOptions() {
	}

	function model($modelName, $isConnection = false) {
		$page = 1;
		$limit = 10; 
		$modelName = camelToUnderscore($modelName, '_');
		
		if (!$isConnection) {
			$model = model($modelName);

			if ($model) {
				if ($this->returnIsList) {
					$data = $model->getAll($this->args + ['site_id' => 1, 'start' => 0, 'limit' => 10]);
					$data = $data[$modelName] ?? [];
				} else {
					$data = $model->get($args + ['site_id' => 1]);
				}
			} else {
				throw new \Exception('Model does not exist!');
			}
		} else {
			$model = model($modelName);

			if ($model) {
				$return   = $model->getAll($this->args + ['site_id' => 1, 'start' => 0, 'limit' => 10]);
				$nodes    = $return[$modelName] ?? [];
				$pageInfo = [
					'count'       => $return['count'] ?? 0,
					'page'        => $page,
					'limit'       => $limit,
					'hasNextPage'     => true,
					'hasPreviousPage' => true,
					'startCursor'     => 1,
				];
				//$nodes = (object) $nodes;

				$data = ['nodes' => $nodes, 'pageInfo' => $pageInfo];
			} else {
				throw new \Exception('Model does not exist!');
			}
		}
		
		return $data;
	}
	
	function defaultFieldResolver($objectValue, $args, $context, ResolveInfo $info) {
		$fieldName = $info->fieldName;

		if (is_null($fieldName)) {
			throw new \Exception('Could not get $fieldName from ResolveInfo');
		}

		if (is_null($info->parentType)) {
			throw new \Exception('Could not get $parentType from ResolveInfo');
		}

		$parentTypeName = $info->parentType->name; //MutationType
		$modelName      = $info->fieldDefinition->config['type']->name ?? '';

		$returnType   = '';
		$this->returnIsList = false;
		$isConnection = false;
		$this->args = $args;

		if ($info->returnType) {
			//method_exists($info->returnType, 'getInnermostType')
			if (is_a($info->returnType, 'GraphQL\Type\Definition\ListOfType')) {
				$returnType   = $info->returnType->getInnermostType()->name;
				$this->returnIsList = true;
			} else {
				$returnType = $info->returnType->name ?? '';
			}
		}

		$method     = '';
		$model      = '';
		$permission = $fieldName;

		if (preg_match('/^([a-z]+)([A-Z]\w+)/', $fieldName, $matches)) {
			$method     = $matches[1];
			$model      = strtolower(camelToUnderscore($matches[2],'_'));
			$permission = "$model/$method";
		}

		$this->permission($permission);

		$data = [];

		if ($fieldName == 'review') {
			return [['content'=>'asdasd', 'author' => 'asdasd']];
		}



		foreach ([$modelName, $returnType] as $type) {
			if (substr_compare($type, 'Type', -4 ,4) === 0) {
				//$modelName = strtolower(preg_replace('/Type$/', '', $modelName));
				$type = substr($type, 0, -4);
				$data = $this->model($type);
				break;
			} else {
				if (substr_compare($type, 'Connection', -10 ,10) === 0) {
					//$modelName = strtolower(preg_replace('/Type$/', '', $modelName));
					$isConnection = true;
					$type    = substr($type, 0, -10);
					$data = $this->model($type, true);
					break;
				}
			}
		}
			

		if ($parentTypeName == 'RootQueryType') {
			$this->queryIndex++;
			
		} else {
			if ($parentTypeName == 'MutationType') {
				$modelName = $fieldName;
				$data      = 1; //[];
				//crud
				if (preg_match('/^(get|add|edit|delete)([A-Z]\w+)/', $modelName, $matches)) {
					$method = $matches[1];
					$model  = $matches[2];

					//return $matches;

					if (isController($model)) {
						$controller = controller($model);

						if ($controller) {
							if (method_exists($controller, $method)) {
								$data = $controller->$method($args + ['site_id' => 1]);
							} else {
								throw new \Exception('Controller method does not exist!');
							}
						} else {
							throw new \Exception('Controller can\'t be loaded!');
						}
					} else {
						$model = strtolower(camelToUnderscore($matches[2],'_'));

						if (isModel($model)) {
							$model = model($model);

							if ($model) {
								if (method_exists($model, $method)) {
									$data = $model->$method($args + ['site_id' => 1]);
								} else {
									throw new \Exception('Model method does not exist!');
								}
							} else {
								throw new \Exception('Model can\'t be loaded!');
							}
						} else {
							throw new \Exception('Model does not exist!');
						}
					}
				}
			}
		}

		$this->stack[$this->queryIndex][$this->stackIndex++] = [
			'model'  => $modelName,
			'method' => $method,
			'args'   => $args,
		];

		if ($data) {
			if ($isConnection) {
				return $data;
			} else {
				return $data;
			}
		}

		$property = null;

		if (is_array($objectValue) || $objectValue instanceof ArrayAccess) {
			if (isset($objectValue[$fieldName])) {
				$property = $objectValue[$fieldName];
			}
		} elseif (is_object($objectValue)) {
			if (isset($objectValue->{$fieldName})) {
				$property = $objectValue->{$fieldName};
			}
		}

		return $property instanceof Closure
		? $property($objectValue, $args, $contextValue, $info)
		: $property;
	}

	function index() {
		$rawInput = file_get_contents('php://input');

		$typeConfigDecorator = function (array $typeConfig, $typeDefinitionNode) : array {
			$name = $typeConfig['name'];
			if ($name == 'ObjectType') {
			}
			// ... add missing options to $typeConfig based on type $name
			return $typeConfig;
		};

		$cacheFilename = DIR_STORAGE . 'model' . DS . 'cached_schema.php';

		if (! file_exists($cacheFilename)) {
			$schemaController = new Schema();
			$schema           = $schemaController->schema(); //file_get_contents(DIR_APP . 'schema.gql');
			$document         = Parser::parse($schema);
			file_put_contents($cacheFilename, "<?php\nreturn " . var_export(AST::toArray($document), true) . ";\n");
		} else {
			$document = AST::fromArray(require $cacheFilename); // fromArray() is a lazy operation as well
		}

		//$typeConfigDecorator = function () {};
		//$schema = BuildSchema::build($document);//, $typeConfigDecorator);
		$this->schema = BuildSchema::build($document, $typeConfigDecorator);

		$input          = json_decode($rawInput, true);
		$query          = $input['query'] ?? '';
		$variableValues = $input['variables'] ?? null;

		try {
			$result    = GraphQL::executeQuery($this->schema, $query, null, null, $variableValues, null, [$this, 'defaultFieldResolver'] /*'Vvveb\Controller\defaultFieldResolver'*/);
			$flags     = 0;

			if (DEBUG) {
				$flags = DebugFlag::INCLUDE_DEBUG_MESSAGE;
			}
			$output    = $result->toArray($flags);
		} catch (\Exception $e) {
			$output = [
				'errors' => [
					[
						'message' => $e->getMessage(),
					],
				],
			];
		}

		return $output;
	}
}
